//快速排序函数
//    要点：
//        1 排序过程中的置换性和排序完成后的有序性
//        2 递归函数的断言处理
//        3 函数的幽灵参数（形参、实参）
//        4 函数协议的多命名行为

#include<limits.h>
#define LENGTH 10000

typedef int array[LENGTH];

int low = 0; //@ strong global invariant init1: low == 0;  // 类型不变式
int up = LENGTH -1; //@ strong global invariant init2: up == LENGTH -1;

/*@
inductive permut(array t1, array t2) =
    (\forall int i:[0..\length(t1)-1].t1[i] == t2[i]) ||
    (\exists int i:[0..\length(t1)-1], int j:[0..\length(t1)-1].
        i <= j &&
    (\forall int k:[0..i-1].t1[k] == t2[k]) && (\forall int k:[i+1..j-1].t1[k] == t2[k]) &&    
    (\forall int k:[j+1..\length(t1)-1].t1[k] == t2[k]) && t1[i] == t2[j] && t1[j] == t2[i]) ||
    (\exists array t3.permut(t1, t3) && permut(t3, t2));
*/

//@ ghost array olda;

/*@                    // 本函数只有在low1 < up1时，才会被quick_sort调用。
    requires 
        low <= low1 && low1 < up1 && up1 <= up 
        && \length(arr) == LENGTH && \offset(arr) == 0 && permut(olda, arr) 
        && (\forall int i:[low1..up1]. min <= arr[i])
        && (\forall int i:[low1..up1]. arr[i] <= max)
        ;
    assigns arr[low1..up1];        
    ensures 
        low <= low1 && low1 < up1 && up1 <= up && low1 <= \result && \result <= up1 
        && \length(arr) == LENGTH && \offset(arr) == 0 && permut(olda, arr) 
        && (\forall int i:[low1..\result-1].arr[i] <= arr[\result]) 
        && (\forall int i:[\result+1..up1].arr[i] > arr[\result])
        && (\forall int i:[low1..up1]. min <= arr[i])
        && (\forall int i:[low1..up1]. arr[i] <= max)
        ;
*/
int partition(int* const arr, const int low1, const int up1)/*@ ghost (int const min, int const max) */
{
    int j, m, ret; int k, temp;

    k = arr[up1];
    m = low1-1;
    j = low1;
    /*@ loop invariant
        low <= low1 < up1 <= up && low1-1 <= m <= up1-1 && low1 <= j <= up1 && m < j 
        && arr[up1] == k && \length(arr) == LENGTH && permut(olda, arr) && \offset(arr) == 0 
        && (\forall int i:[low1..m]. arr[i] <= k) && (\forall int i:[m+1..j-1]. arr[i] > k)
        && (\forall int i:[low1..up1]. min <= arr[i])
        && (\forall int i:[low1..up1]. arr[i] <= max)
        ;
    */
    while(j != up1){
        if(arr[j] <= k){
            m = m + 1;
            temp = arr[m];
            arr[m] = arr[j];
            arr[j] = temp;
        }
        j = j + 1;    
    }
    temp = arr[up1];
    arr[up1] = arr[m+1];
    arr[m+1] = temp;
    ret = m+1;
    return ret;
}


/*@
    requires 
        low <= low2 <= up+1 && low-1 <= up2 <= up && \length(arr) == LENGTH && \offset(arr) == 0 
        && permut(olda, arr) 
        && (\forall int i:[0..LENGTH-1].\old(arr[i])==arr[i])
        && (\forall int i:[low2..up2]. min <= arr[i])
        && (\forall int i:[low2..up2]. arr[i] <= max)
        ;
    decreases up2-low2+1;
    ensures 
        \length(arr) == LENGTH && \offset(arr) == 0 
        && (\forall int i:[low2..up2]. min <= arr[i])
        && (\forall int i:[low2..up2]. arr[i] <= max)
        ;

    behavior less_than_two_elements:
        assumes low2 >= up2;
        ensures 
            permut(olda, arr) && \forall int i:[0..LENGTH-1].\old(arr[i])==arr[i] ;

    behavior less_then:            
        assumes low2 < up2;
        assigns arr[low2..up2];
        ensures 
            low <= low2 < up2 <= up && permut(olda, arr) 
            && (\forall int i:[low2..up2-1].arr[i] <= arr[i+1]);
    complete behaviors;
    disjoint behaviors;
*/
void quick_sort(int* const arr, const int low2, const int up2)/*@ ghost (int const min, int const max) */
{
    int i;

    if(low2 < up2){
        i = partition(arr, low2, up2) /*@ ghost (min, max) */;    
        quick_sort(arr, low2, i - 1)  /*@ ghost (min, arr[i]) */;    
        quick_sort(arr, i + 1, up2)   /*@ ghost (arr[i], max) */;    
    }
    return;
}

/*@ 
    requires 
        low == 0 && up == LENGTH -1 && \length(a)==LENGTH && \offset(a)==0 ;
    assigns olda,a[0..LENGTH -1];
    ensures 
        (\forall int i: [low..up-1]. a[i] <= a[i+1]) && \length(a)==LENGTH && \offset(a)==0
        && \result == 0 && permut(olda, a) 
        ;
*/
int gosorting(int* const a){
    //@ ghost olda[0..LENGTH-1] = a[0..LENGTH-1];
    quick_sort(a, low, up)/*@ ghost (INT_MIN, INT_MAX) */; // 幽灵实参
    return 0;
}